# coding: utf-8
from icu import RuleBasedCollator
from pathlib import Path


rules = Path(__file__).parent / "../third_party/rules.txt"
collator = RuleBasedCollator(
    '[normalization on]\n[reorder Tibt]\n' +
    rules.read_text(encoding="utf-8")
)


def bo_sorted(word_list):
    return sorted(word_list, key=collator.getSortKey)




import click
from pathlib import Path

from botok import Text, __version__, rdr_2_replace_matcher
from pybo import get_regex_pairs, batch_apply_regex, bo_sorted


@click.group()
@click.version_option(__version__)
def cli():
    pass


# Tokenize file
@cli.command()
@click.argument("filename", type=click.Path(exists=True))
def tok_file(**kwargs):
    click.echo(f'parsing {kwargs["filename"]}...')
    t = Text(Path(kwargs["filename"]))
    t.tokenize_words_raw_lines
    click.echo("done")


# Tokenize string
@cli.command()
@click.argument("string")
def tok_string(**kwargs):
    t = Text(kwargs["string"])
    click.echo(t.tokenize_words_raw_lines)


# rdr_2_replace_matcher
@cli.command()
@click.argument("infile", type=click.Path(exists=True))
def rdr2repl(**kwargs):
    infile = Path(kwargs["infile"])
    outfile = infile.parent / (infile.stem + ".yaml")
    dump = infile.read_text(encoding="utf-8-sig")
    processed = rdr_2_replace_matcher(dump)
    outfile.write_text(processed, encoding="utf-8-sig")


# sort in the Tibetan order
@cli.command()
@click.argument("infile", type=click.Path(exists=True))
def kakha(**kwargs):
    infile = Path(kwargs["infile"])
    words = infile.read_text(encoding="utf-8-sig").split()
    words = bo_sorted(words)
    infile.write_text("\n".join(words), encoding="utf-8-sig")


# FNR - Find and Replace with a list of regexes
@cli.command()
@click.argument("in-dir", type=click.Path(exists=True))
@click.argument("regex-file", type=click.Path(exists=True))
@click.option("-o", "--out-dir", type=click.Path())
@click.option("-t", "--tag")
def fnr(**kwargs):
    # get the args
    indir = Path(kwargs["in_dir"])
    regex_file = Path(kwargs["regex_file"])
    out_dir = Path(kwargs["out_dir"]) if kwargs["out_dir"] else None

    if not indir.is_dir():
        click.echo("IN_DIR should be a folder, not a file.\nexiting...")
        exit(1)

    # optional out file tag
    tag = kwargs["tag"] if kwargs["tag"] else regex_file.stem

    # generate rules
    rules = get_regex_pairs(regex_file.open(encoding="utf-8-sig").readlines())

    # apply on each file, prefixing each one with the regex filename
    for f in indir.rglob("*.txt"):
        if not f.stem.startswith("_"):
            string = f.read_text(encoding="utf-8-sig")
            out = batch_apply_regex(string, rules)
            name = f"_{tag}_" + f.name
            if out_dir:
                Path(out_dir).mkdir(parents=True, exist_ok=True)
                outfile = out_dir / name
            else:
                outfile = f.parent / name
            outfile.write_text(out, encoding="utf-8-sig")


if __name__ == "__main__":
    cli()



#! /usr/bin/env python
# -*- coding: utf8 -*-

from __future__ import print_function

from pathlib import Path
import setuptools
from pkg_resources import parse_version

assert parse_version(setuptools.__version__) >= parse_version("38.6.0")


def read(fname):
    p = Path(__file__).parent / fname
    with p.open(encoding="utf-8") as f:
        return f.read()


setuptools.setup(
    name="pybo",
    version="0.6.11",  # also edit version in pybo/__init__.py
    author="Esukhia development team",
    author_email="esukhiadev@gmail.com",
    description="Python utils for processing Tibetan",
    license="Apache2",
    keywords="nlp computational_linguistics search ngrams language_models linguistics toolkit tibetan",
    url="https://github.com/Esukhia/pybo",
    packages=setuptools.find_packages(),
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    project_urls={
        "Source": "https://github.com/Esukhia/pybo",
        "Tracker": "https://github.com/Esukhia/pybo/issues",
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Text Processing :: Linguistic",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: Tibetan",
    ],
    package_data={"pybo": ["third_party/*"]},
    python_requires=">=3.6",
    tests_require=["pytest"],
    install_requires=["pyyaml", "click", "botok", "pyewts"],
    entry_points={
        "console_scripts": ["pybo=pybo.cli:cli"]  # command=package.module:function
    },
)



# coding: utf-8
import re


def batch_apply_regex(string, pairs):
    for find, repl in pairs:
        string = re.sub(find, repl, string, flags=re.MULTILINE)
    return string


def get_regex_pairs(lines, sep="\t-\t"):
    regex_pairs = []
    clean_lines = _parse_lines(lines, sep)

    for line in clean_lines:
        find, replace = line.split(sep)
        regex_pairs.append((r"" + find, r"" + replace))
    return regex_pairs


def _parse_lines(lines, sep):
    cleaned = []
    for num, line in enumerate(lines):
        # remove comment lines and empty lines
        if "#" in line:
            line = line[: line.find("#")]

        # strip line returns while keeping space chars and screen all empty lines
        line = line.strip("\n\r")
        if not line:
            continue

        # ensure there is 1 and only 1 occurrence of sep
        if line.count(sep) != 1:
            print(f"passing line {num + 1}: {line}.")
            continue

        cleaned.append(line)
    return cleaned
